<?php 

// +------------------------------------------------------------------+
// | Package Information                                              |
// +------------------------------------------------------------------+

/**
 * @package     EuropaCart
 * @subpackage  Components
 * @copyright   Copyright 1999-Present Kryptronic, Inc.  All rights reserved.
 * @link        http://www.kryptronic.com
 * @license     GNU/GPL
*/

// +------------------------------------------------------------------+
// | Package                                                          |
// +------------------------------------------------------------------+

// +--
// | No direct access.
// +--

defined('_JEXEC') or die ('Restricted access');

// +--
// | Import.
// +--

jimport( 'joomla.application.component.model' );

// +------------------------------------------------------------------+
// | Class Definition                                                 |
// +------------------------------------------------------------------+

class EuropaCartModelEuropaCart extends JModel {

// +--
// | EuropaCartCommon class variable.
// +--

var $EuropaCartCommon;

// +------------------------------------------------------------------+
// | Function: getOutput                                              |
// +------------------------------------------------------------------+

function getOutput () {

// +--
// | This function handles all of the FrontEnd work for this component.
// +--

// +--
// | Load the EuropaCartCommon class from the EuropaCart_ObjectCache
// | so we can access common libraries.
// +--

global $EuropaCart_ObjectCache;

$this->EuropaCartCommon =& $EuropaCart_ObjectCache['EuropaCartCommon'];

// +--
// | Return the connector output.
// +--

return $this->connector();

} // End of function.

// +------------------------------------------------------------------+
// | Function: connector                                              |
// +------------------------------------------------------------------+

function connector () {

// +--
// | This function provides connector functionality via an XML interface
// | for EuropaCart.
// +--

global $EuropaCart_ConnectionActive;

$EuropaCart_ConnectionActive = 1;

// +--
// | Return right away if our connector is not working.
// +--

if (empty($this->EuropaCartCommon)) {

     return 'The EuropaCart Connector for Joomla! is not properly configured.  Please ensure the EuropaCart System Module for Joomla! is installed and activated.';

} // End of if statement.

// +--
// | Handle the request and error out if the 'request' CGI variable
// | is not populated.
// +--

$request = '';

if (!(empty($_POST['request']))) {$request = $_POST['request'];}
if (!(empty($_GET['request'])))  {$request = $_GET['request'];}

if (empty($request)) {

     return $this->connector_response('ERROR','The CGI request received did not contain a request parameter.');

} // End of if statement.

$request = $this->EuropaCartCommon->decrypt($request,$this->EuropaCartCommon->EuropaCartPWD);

// +--
// | Parse the request and error out if it is not properly formatted.
// +--

$xmlrequest = $this->EuropaCartCommon->parseXML($request);

if (empty($xmlrequest['xml.0'])) {

     return $this->connector_response('ERROR','The XML request received was missing required nodes.');

} // End of if statement.

// +--
// | Work with the request and error out if the required nodes are not complete.
// +--

$password  = '';
$app       = '';
$namespace = '';
$params    = '';

if (!(empty($xmlrequest['xml.0']['password.0'])))  {$password  = $xmlrequest['xml.0']['password.0'];}
if (!(empty($xmlrequest['xml.0']['app.0'])))       {$app       = $xmlrequest['xml.0']['app.0'];}
if (!(empty($xmlrequest['xml.0']['namespace.0']))) {$namespace = $xmlrequest['xml.0']['namespace.0'];}
if (!(empty($xmlrequest['xml.0']['params.0'])))    {$params    = @unserialize($xmlrequest['xml.0']['params.0']);}

if ((empty($password)) || (empty($app)) || (empty($namespace))) {

     return $this->connector_response('ERROR','The XML request received was missing required nodes.');

} // End of if statement.

// +--
// | Check the password to see if it's valid and error out if it isn't.
// +--

if ($password != $this->EuropaCartCommon->EuropaCartPWD) {

     return $this->connector_response('ERROR','The XML request received contained an invalid connector password.');

} // End of if statement.

// +--
// | Execute the function requested.  Setting $_REQUEST['tmpl'] to 'component'
// | suppresses the normal skin and includes a limited 'component' skin.
// +--

if ($namespace == 'joomlasinglesignon') {

     $_REQUEST['tmpl'] = 'component';

     $response = $this->singlesignon($params);

     return $this->connector_response($response['status'],$response['response']);

} elseif ($namespace == 'joomlaskinimporter') {

     $response = $this->skinimporter();

     return $this->connector_response($response['status'],$response['response']);

} else {

     $_REQUEST['tmpl'] = 'component';

     return $this->connector_response('ERROR','The XML request received contained an invalid namespace reference.');

} // End of if statement.

} // End of function.

// +------------------------------------------------------------------+
// | Function: connector_response                                     |
// +------------------------------------------------------------------+

function connector_response ($status = 'OK', $response = '') {

// +--
// | Prints an XML response.
// +--

$eol          = "\r\n";

$xmlresponse  = '<!DOCTYPE xml SYSTEM "http://central.kryptronic.com/public/dtd/';
$xmlresponse .= 'joomla_connector_resp_v1.0.dtd">' . $eol;

// +--
// | Build out an XML array and formulate it as XML.
// +--

$xmlbuild['xml.0']['status.0']   = $status;
$xmlbuild['xml.0']['response.0'] = $response;

$result = $this->EuropaCartCommon->buildXML($xmlbuild);

$xmlresponse .= $result;

$xmlresponse  = $this->EuropaCartCommon->encrypt($xmlresponse,$this->EuropaCartCommon->EuropaCartPWD);

// +--
// | Return the response.
// +--

return $xmlresponse;

} // End of function.

// +------------------------------------------------------------------+
// | Function: skinimporter                                           |
// +------------------------------------------------------------------+

function skinimporter () {

// +--
// | There is no magic to this function.  We don't override the skin
// | if this method is called, so we simply present an OK response and
// | it will be wrapped in the Joomla skin.
// +--

// +--
// | Return a response.
// +--

return array('status' => 'OK', 'response' => 'Skin included.');

} // End of function.

// +------------------------------------------------------------------+
// | Function: singlesignon                                           |
// +------------------------------------------------------------------+

function singlesignon ($params = array()) {

// +--
// | This function handles single-sing-on tasks for Joomla!.
// +--

// +--
// | Load some modules and a database object.
// +--

jimport('joomla.user.helper');
jimport('joomla.utilities.utility');

$db         =& JFactory::getDBO();

JPluginHelper::importPlugin('user');

$dispatcher =& JDispatcher::getInstance();

// +--
// | Set up the response array.
// +--

$response = array('status'   => 'ERROR',
                  'response' => 'Initial response.');

// +--
// | Check parameters to make sure they're complete.
// +--

if (empty($params['method'])) {

     $response = array('status'   => 'ERROR',
                       'response' => 'Parameters were not included in the request.');

     return $response;

} // End of if statement.

$method = $params['method']; unset($params['method']);

// +--
// | Handle the various methods.
// |
// | METHOD: user_logout
// +--

if ($method == 'user_logout') {

     $id = $this->get_userid($params['user']['id']);

     if (!(empty($id))) {

          $my =& JFactory::getUser();

          if ($id != 0) {

               $sql = 'SELECT * FROM #__users WHERE id=' . (int) $id;

               $db->setQuery($sql);

               $result = $db->loadAssocList();

               $result = (is_array($result[0])) ? $result[0] : array();

               if (!(empty($result))) {

                    $dispatcher->trigger('onLogoutUser', array($result));

               } // End of if statement.

               if ($my->get('id') == $id) {

                    $my->setLastVisit();

                    $session =& JFactory::getSession();
                    $session->destroy();

               } else {

                    $table = & JTable::getInstance('session');
                    $table->destroy($id,'');

               } // End of if statement.

          } // End of if statement.

     } // End of if statement.

     $response = array('complete' => 1);

     $response = array('status'   => 'OK',
                       'response' => @serialize($response));

// +--
// | METHOD: user_getinfo
// +--

} elseif ($method == 'user_getinfo') {

     $id = $this->get_userid($params['user']['id']);

     $sql = 'SELECT * FROM #__users WHERE id=' . (int) $id;

     $db->setQuery($sql);

     $result = $db->loadAssocList();

     $result = (is_array($result[0])) ? $result[0] : array();

     if (!(empty($result))) {

          $response = $this->modify_userdata($result);

          $response['complete'] = 1;      

          $response = array('status'   => 'OK',
                            'response' => @serialize($response));

     } else {

          $response = array('status'   => 'ERROR',
                            'response' => 'User account does not exist.');

     } // End of if statement.

// +--
// | METHOD: user_saveinfo
// +--

} elseif ($method == 'user_saveinfo') {

     $this->store_userdata($params,$method);

     $response = array('complete' => 1);

     $response = array('status'   => 'OK',
                       'response' => @serialize($response));

// +--
// | METHOD: user_login
// +--

} elseif ($method == 'user_login') {

     $this->store_userdata($params,$method);

     $response = array('complete' => 1);

     $response = array('status'   => 'OK',
                       'response' => @serialize($response));

// +--
// | METHOD: user_auth
// +--

} elseif ($method == 'user_auth') {

     $id = $this->get_userid($params['user']['id']);

     $sql = 'SELECT * FROM #__users WHERE id=' . (int) $id;

     $db->setQuery($sql);

     $result = $db->loadAssocList();

     $result = (is_array($result[0])) ? $result[0] : array();

     $authorized = 0;

     if (!(empty($result))) {

          $credentials = array('password' => $params['user']['password'],
                               'username' => $result['username'],
                               'email'    => $result['email']);

          jimport( 'joomla.user.authentication');

          $authenticate =& JAuthentication::getInstance();
          $response	=  $authenticate->authenticate($credentials);

          if ($response->status === JAUTHENTICATE_STATUS_SUCCESS) {$authorized = 1;}

     } // End of if statement.

     if (!(empty($authorized))) {

          $response = $this->modify_userdata($result);

          $response['complete'] = 1;

          $response = array('status'   => 'OK',
                            'response' => @serialize($response));

     } else {

          $response = array('status'   => 'ERROR',
                            'response' => 'User account could not be authenticated.');

     } // End of if statement.

// +--
// | METHOD: user_changeid
// +--

} elseif ($method == 'user_changeid') {

     $this->store_userdata($params,$method);

     $response = array('complete' => 1);

     $response = array('status'   => 'OK',
                       'response' => @serialize($response));

// +--
// | METHOD: user_changeusername
// +--

} elseif ($method == 'user_changeusername') {

     $this->store_userdata($params,$method);

     $response = array('complete' => 1);

     $response = array('status'   => 'OK',
                       'response' => @serialize($response));

// +--
// | METHOD: INVALID
// +--

} else {

     $response = array('status'   => 'ERROR',
                       'response' => 'The method node included in the request was invalid.');

} // End of if statement.

// +--
// | Return the response.
// +--

return $response;

} // End of function.


// +------------------------------------------------------------------+
// | Function: get_userid                                             |
// +------------------------------------------------------------------+

function get_userid ($identifier = '') {

// +--
// | Returns the user id for a username or email.
// +--

$db =& JFactory::getDBO();

$id = '';

if (empty($identifier)) {return $id;}

// +--
// | Get the id from the database.
// +--

$sql  = 'SELECT id FROM #__users WHERE username=' . $db->Quote($identifier);
$sql .= ' OR email=' . $db->Quote($identifier);

$db->setQuery($sql);

$result = $db->loadObject();

if ($result) {$id = $result->id;}

// +--
// | Return the id.
// +--

return $id;

} // End of function.

// +------------------------------------------------------------------+
// | Function: modify_userdata                                        |
// +------------------------------------------------------------------+

function modify_userdata ($userdata = array()) {

// +--
// | This function takes a Joomla user db array and transforms it
// | into something EuropaCart can work with.
// +--

if (!(empty($userdata))) {

     if (!(empty($userdata['id'])))    {unset($userdata['id']);}
     if (!(empty($userdata['email']))) {$userdata['id'] = $userdata['email']; unset($userdata['email']);}
     if (empty($userdata['id']))       {$userdata['id'] = '';}

     foreach ($userdata as $key => $data) {

          if ($key == 'name') {

               if (preg_match('/ /',$data)) {

                    list($fname,$lname) = split(' ', $data, 2);

               } else {

                    $fname = ' ';
                    $lname = $data;

               } // End of if statement.

               $userdata['fname'] = $fname;
               $userdata['lname'] = $lname;

               unset($userdata['name']);

          } elseif ($key == 'password') {

               $userdata[$key] = '';

          } else {

               $userdata[$key] = $data;

          } // End of if statement.

     } // End of foreach statement.

     $userdata['usergroup'] = 'users';

} // End of if statement.

// +--
// | Return the user data.
// +--

return $userdata;

} // End of function.

// +------------------------------------------------------------------+
// | Function: store_userdata                                         |
// +------------------------------------------------------------------+

function store_userdata ($params = array(),$method = '') {

// +--
// | Load some modules and a database object.
// +--

jimport('joomla.user.helper');
jimport('joomla.utilities.utility');

$db         =& JFactory::getDBO();


JPluginHelper::importPlugin('user');

$dispatcher =& JDispatcher::getInstance();

// +--
// | Get info on the id from the database and get a user instance.
// +--

$id = $this->get_userid($params['user']['id']);

$sql = 'SELECT * FROM #__users WHERE id=' . (int) $id;

$db->setQuery($sql);

$result = $db->loadAssocList();

$result = (is_array($result[0])) ? $result[0] : array();

$instance = new JUser();

if (!(empty($result))) {$instance->load($result['id']);} else {$id = null;}

// +--
// | Set some default values.
// +--

jimport('joomla.application.component.helper');

$config   =& JComponentHelper::getParams( 'com_users' );
$acl      =& JFactory::getACL();

$usersconfig = &JComponentHelper::getParams( 'com_users' );
$newusertype = $usersconfig->get('new_usertype'); if (!$newusertype) {$newusertype = 'Registered';}

if (!(empty($result['usertype']))) {$usertype =  $result['usertype'];}
else                               {$usertype =  $newusertype;}

if ($acl->is_group_child_of($usertype,'Registered') || $acl->is_group_child_of($usertype,'Public Backend')) {$aid = 2;}
else                                                                                                        {$aid = 1;}

$set_email    = $params['user']['id'];       if (!(empty($params['user']['newid'])))       {$set_email    = $params['user']['newid'];}
$set_username = $params['user']['username']; if (!(empty($params['user']['newusername']))) {$set_username = $params['user']['newusername'];}

// +--
// | Determine whether the user is new and fire the onBeforStoreUser event.
// +--

$isnew = 0; if (empty($id)) {$isnew = 1;}

$dispatcher->trigger('onBeforeStoreUser', array($instance->getProperties(),$isnew));

// +--
// | Set user instance properties.
// +--

$instance->set( 'id', $id);
$instance->set( 'name', $params['user']['fname'] . ' ' . $params['user']['lname'] );
$instance->set( 'username', $set_username );
$instance->set( 'email', $set_email );
$instance->set( 'gid', $acl->get_group_id( '', $usertype));
$instance->set( 'usertype', $usertype );
$instance->set( 'guest', 0);
$instance->set( 'aid', $aid);
$instance->set( 'block', 0);

// +--
// | Set the password if we have it a clear text and rn the onLoginUser
// | event if we have a user_login request.
// +--

if (!(empty($params['user']['password_clear']))) {

     $salt     = JUserHelper::genRandomPassword(32);
     $crypt    = JUserHelper::getCryptedPassword($params['user']['password_clear'], $salt);
     $password = $crypt.':'.$salt;

     $instance->set( 'password', $password);
     $instance->set( 'password_clear', $params['user']['password_clear']);

     if ($method == 'user_login') {

          $credentials = array('password' => $params['user']['password_clear'],
                               'username' => $params['user']['username'],
                               'email'    => $params['user']['id']);

          jimport( 'joomla.user.authentication');

          $authenticate =& JAuthentication::getInstance();
          $response     =  $authenticate->authenticate($credentials);

          if ($response->status === JAUTHENTICATE_STATUS_SUCCESS) {

               $dispatcher->trigger('onLoginUser', array((array)$response,array()));

          } // End of if statement.

     } // End of if statement.

} // End of if statement.

// +--
// | Save the user instance and trigger the onAfterStoreUser event.  Also
// | conditionally handle some session stuff.
// +--

$instance->save();

$dispatcher->trigger('onAfterStoreUser', array($instance->getProperties(),$isnew,1));

if (empty($id)) {$id = $this->get_userid($params['user']['id']);}

if ($params['user']['update_session']) {

     if ($id != 0) {

          $session =& JFactory::getSession();

          $session->set('user', $instance);

          $table = & JTable::getInstance('session');
          $table->load( $session->getId() );
          $table->guest    = $instance->get('guest');
          $table->username = $instance->get('username');
          $table->userid   = intval($instance->get('id'));
          $table->usertype = $instance->get('usertype');
          $table->gid      = intval($instance->get('gid'));

          $table->update();

     } // End of if statement.

} // End of if statement.

$instance->setLastVisit();

// +--
// | Return true.
// +--

return 1;

} // End of function.

// +------------------------------------------------------------------+
// | End of Class                                                     |
// +------------------------------------------------------------------+

} // End of class.

// +------------------------------------------------------------------+
// | End Of File                                                      |
// +------------------------------------------------------------------+

?>